(function() {

// constructor
function GameEngine(stage, width, height, fps) {
    this.stage = stage;
    GameEngine.W = width;
    GameEngine.H = height;
    GameEngine.fps = fps;
    GameEngine.engine = this;
    this._scenes = new Array();
    this._currentScene = null;
    this._showFPS = false;
}
var p = GameEngine.prototype;

// public methods:
p.tick = function(e) {
    if (this._currentScene.sceneReady) {
        //if (document.getElementById('cover').style.visibility == 'hidden') {
            this._currentScene.tick(e);
        //}
        if (this._showFPS) {
            this._fpsLabel.text = Math.round(createjs.Ticker.getMeasuredFPS()) + ' fps';
        }
    }
    this.stage.update();
};

p.start = function() {
    //start ticking
    createjs.Ticker.on("tick", this.tick, this);
    createjs.Ticker.setFPS(GameEngine.fps);
};

p.addScene = function(scene) {
    this._scenes.push(scene);
};

p.setScene = function(newScene) {
    if (typeof newScene == "string") {
        for (var i = 0; i < this._scenes.length; i++) {
            if (this._scenes[i].id == newScene) {
                newScene = i; break;
            }
        }
    }
    
    //remove current scene
    if (this.stage.contains(this._currentScene)) {
        this.stage.removeChild(this._currentScene);
    }
    
    //set new current scene
    this._currentScene = this._scenes[newScene];
    this.stage.addChild(this._currentScene);
    
    if (this._currentScene.sceneReady) {
        //console.log("enter scene from set scene on engine");
        this._currentScene.enterScene();
    }
    
    if (this._showFPS) {
        this.stage.addChild(this._fpsLabel);
    }
};

p.enableMouse = function() {
    this.stage.enableMouseOver();
};

p.enableTouch = function() {
    createjs.Touch.enable(this.stage);
};

p.enableInput = function() {
    document.onkeydown = this._keyDownListener(this);
    document.onkeyup = this._keyUpListener(this);
};

p.enableResize = function() {
    window.onresize = this._resizeListener(this);
};

p.enablePixelSnap = function() {
    stage.snapPixelsEnabled = true;
    createjs.DisplayObject.prototype.snapToPixel = true;
};

p.showFPS = function() {
    this._showFPS = true;
    this._fpsLabel = new createjs.Text("-- fps","bold 12px Arial","#999");
    this._fpsLabel.x = 6;
    this._fpsLabel.y = 6;
};

p._keyDownListener = function(that){
    return function(e){
        if(!e){ var e = window.event; } //cross browser issue
        //e.preventDefault();
        that._currentScene.handleKeyDown(e);
    };
};

p._keyUpListener = function(that){
    return function(e){
        if(!e){ var e = window.event; } //cross browser issue
        //e.preventDefault();
        that._currentScene.handleKeyUp(e);
    };
};

p._resizeListener = function(that){
    return function(e){
        if(!e){ var e = window.event; } //cross browser issue
        
        var verticalScale = (window.innerHeight / that.stage.canvas.height);
        var horizontalScale = (window.innerWidth / that.stage.canvas.width);
        
        if (verticalScale < horizontalScale) {
            that.stage.canvas.style.width = that.stage.canvas.width * verticalScale + 'px';
            that.stage.canvas.style.height = window.innerHeight + 'px';
        }
        else {
            that.stage.canvas.style.width = window.innerWidth + 'px';
            that.stage.canvas.style.height = that.stage.canvas.height * horizontalScale + 'px';
        }
        
        console.log("ENGINE RESIZED");
    };
};

window.GameEngine = GameEngine;
}());