(function() {

// constructor
function GameScene(id, game) {
	this.Container_constructor();
	
    // public properties:
	this.sceneReady = true;
    this.id = id;
    
    // private properties:
	this._game = game;
    
    this._loader = new createjs.LoadQueue();
    this._loader.installPlugin(createjs.Sound);
    this._loader.setMaxConnections(5);
    
    //list of sprite sheets
    this._spriteSheets = new Array();
    
	this.setup();
}
var p = createjs.extend(GameScene, createjs.Container);

p.setup = function() {
    this.snapToPixel = true;
};
    
// public methods:    
p.addManifest = function(manifest) {
    this._loader.loadManifest(manifest);
    
    this.sceneReady = false;
    this._loader.on("complete", this._loadComplete, this);
};

p.getLoadResult = function(name, scale) {
    var image = this._loader.getResult(name);
    
    if (arguments.length == 1) {
        return image;
    }
    else {
        return Utility.nearestNeighborScale(image, scale);
    }
};

p.switchScene = function(newScene) {
    //fade out
    this.alpha = 1.0;
    createjs.Tween.get(this, {loop:false, override:true}).to({alpha:0.0}, 1000, createjs.Ease.cubicInOut).call(this.exitScene, [newScene], this);
};

// private methods:     
p._loadComplete = function() {
    if (!this.sceneReady && this._loader.loaded) {
        this.sceneReady = true;
        
        if (GameEngine.engine._currentScene === this) {
            console.log("enter scene from load complete listener");
            this.enterScene();
        }
    }
};

// abstract methods:
p.tick = function() {};
p.enterScene = function() {};
p.exitScene = function(newScene) { GameEngine.setScene(newScene); };
p.handleKeyDown = function(e) {};
p.handleKeyUp = function(e) {};

window.GameScene = createjs.promote(GameScene, "Container");
}());