(function(window) {

//
function Level() {
  this.initialize();
}
var p = Level.prototype;
// public properties:
    p.width = 0;
    p.height = 0;
    
    p.suspects = null;
    p.murderer = 0;
    p.victimName = 0;
    p.crime = 0;
    p.location = 0;
    
    p.incriminatingEvidence = null;
    p.suspectInformation = null;
    p.eliminationClues = null;
    
    p.searchesAllowed = 0;
    
// private properties
    p._dummy = 0;
    
// constructor:
    p.initialize = function() {
        this.incriminatingEvidence = new Array();
        this.suspectInformation = new Array();
        this.eliminationClues = new Array();
    };
    
// public methods:    
    p.tick = function() {
        
    };
    
    p.commitCrime = function() {
        this.victimName = Utility.random(0, Game.victimNames.length-1);
        this.crime = Utility.random(0, Game.crimes.length-1);
        this.location = Utility.random(0, Game.locations.length-1);
    };
    
    p.defineSuspects = function(suspectCount, traitsPerSuspect) {
        this.suspects = new Array();
        
        var titlesList = _.shuffle(_.range(Game.titles.length));
        var traitsList = _.shuffle(_.range(2, Game.traits.length)).slice(0, suspectCount * traitsPerSuspect + suspectCount);
        
        for (var i = 0; i < suspectCount; i++) {
            var title = titlesList[i];
            var hand = [Utility.random(0, 1)];
            var traits = hand.concat(_.sample(traitsList, traitsPerSuspect));
            
            //make sure no suspects have identical traits
            var duplicate = true;
            var count = 0;
            while (duplicate && count < 1000) {
                duplicate = false;
                count++;
                hand = [Utility.random(0, 1)];
                traits = hand.concat(_.sample(traitsList, traitsPerSuspect));
                
                for (var j = 0; j < i; j++) {
                    var intersectionCount = _.intersection(this.suspects[j].traits, traits);
                    if (intersectionCount == traitsPerSuspect) {
                        duplicate = true;
                        //console.log("DUPLICATE");
                    }
                }
            }
            
            this.suspects.push( {title:title, traits:traits} );
        }
        
        var fail = false;
        
        //count clue overlaps
        var totalIntersections = 0;
        for (var i = 0; i < suspectCount; i++) {
            for (var j = i; j < suspectCount; j++) {
                if (i != j) {
                    var intersections = _.intersection(this.suspects[i].traits, this.suspects[j].traits).length;
                    //console.log('Traits intersection for ' + i + ', ' + j + ': ' + intersections);
                    totalIntersections += intersections;
                    
                    if (intersections == traitsPerSuspect+1) {
                        fail = true;
                        //console.log("ERROR - DUPLICATE NOT REMOVED");
                    }
                }
            }
        }
        
        if (totalIntersections < traitsPerSuspect || fail) {
            //console.log("Need more intersections");
            this.defineSuspects(suspectCount, traitsPerSuspect);
        }
        else {
            this.murderer = Utility.random(0, this.suspects.length-1);
            //console.log('Trait intersections: ' + totalIntersections);
            //console.log(this.suspects);
            //console.log("Murderer: " + this.murderer);
        }
    };
    
    p.placeClues = function() {
        //calculate needed grid size
        var traitsPerSuspect = this.suspects[0].traits.length;
        var suspectCount = this.suspects.length;
        var incriminatingEvidenceCount = traitsPerSuspect * 2;
        var suspectInformationCount = traitsPerSuspect * suspectCount * 3;
        var eliminationClueCount = (traitsPerSuspect * (suspectCount-1)) * 4;
        var minimumClueSpaceNeeded = incriminatingEvidenceCount + suspectInformationCount + eliminationClueCount;
        
        //set grid size
        this.width = Math.floor(Math.sqrt(minimumClueSpaceNeeded * 2.0));
        this.height = this.width;
        
        //place clues in grid
        this.addIncriminatingEvidence();
        this.addSuspectInformation();
        this.addEliminationClues();
        
        //console.log(this.incriminatingEvidence);
        //console.log(this.suspectInformation);
        //console.log(this.eliminationClues);
        
        //set allowed searches
        //this.searchesAllowed = Math.round((this.width * this.height) / 2);
        this.searchesAllowed = this.width * this.height;
    };
    
    p.addIncriminatingEvidence = function() {
        for (var i = 0; i < this.suspects[this.murderer].traits.length; i++) {
            var x = Utility.random(0, this.width-2);
            var y = Utility.random(0, this.height-2);
            
            var count = 0;
            while ((this.hasClueAt(x, y) || this.hasClueAt(x, y+1)) && count < 1000) {
                count++;
                x = Utility.random(0, this.width-2);
                y = Utility.random(0, this.height-2);
            }
            
            if (count < 1000) {
                this.incriminatingEvidence.push( {x:[x, x], y:[y, y+1], pieces:2, found:0, trait:this.suspects[this.murderer].traits[i], suspect:this.murderer} );
            }
            else {
                console.log("INCRIMINATING EVIDENCE MISSING");
            }
        }
    };
    
    p.addSuspectInformation = function() {
        for (var j = 0; j < this.suspects.length; j++) {
            for (var i = 0; i < this.suspects[j].traits.length; i++) {
                var x = Utility.random(0, this.width-3);
                var y = Utility.random(0, this.height-3);
                
                var count = 0;
                while ((this.hasClueAt(x, y) || this.hasClueAt(x+1, y) || this.hasClueAt(x+2, y)) && count < 100) {
                    count++;
                    x = Utility.random(0, this.width-3);
                    y = Utility.random(0, this.height-3);
                }
                
                if (count < 1000) {
                    this.suspectInformation.push( {x:[x, x+1, x+2], y:[y, y, y], pieces:3, found:0, trait:this.suspects[j].traits[i], suspect:j} );
                }
                else {
                    console.log("SUSPECT INFORMATION MISSING");
                }
            }
        }
    };
    
    p.addEliminationClues = function() {
        var eliminationList = new Array();
    
        for (var j = 0; j < this.suspects.length; j++) {
            if (j != this.murderer) {
                for (var i = 0; i < this.suspects[j].traits.length; i++) {
                    var x = Utility.random(0, this.width-2);
                    var y = Utility.random(0, this.height-2);
                    
                    var count = 0;
                    while ((this.hasClueAt(x, y) || this.hasClueAt(x+1, y) || this.hasClueAt(x, y+1) || this.hasClueAt(x+1, y+1)) && count < 100) {
                        count++;
                        x = Utility.random(0, this.width-2);
                        y = Utility.random(0, this.height-2);
                    }
                    
                    if (count < 1000 && !_.contains(this.suspects[this.murderer].traits, this.suspects[j].traits[i]) && !_.contains(eliminationList, this.suspects[j].traits[i]) ) {
                        this.eliminationClues.push( {x:[x, x+1, x, x+1], y:[y, y, y+1, y+1], pieces:4, found:0, trait:this.suspects[j].traits[i], suspect:j} );
                        eliminationList.push(this.suspects[j].traits[i]);
                    }
                    else {
                        //console.log("ELIMINATION CLUE MISSING OR ALREADY ADDED " + this.suspects[j].traits[i]);
                    }
                }
            }
        }
    };
    
    p.hasClueAt = function(x, y) {
        var clueType = 0;
        
        for (var i = 0; i < this.incriminatingEvidence.length; i++) {
            for (var j = 0; j < this.incriminatingEvidence[i].x.length; j++) {
                if (x == this.incriminatingEvidence[i].x[j] && y == this.incriminatingEvidence[i].y[j]) {
                    clueType = 1;
                }
            }
        }
        
        for (var i = 0; i < this.suspectInformation.length; i++) {
            for (var j = 0; j < this.suspectInformation[i].x.length; j++) {
                if (x == this.suspectInformation[i].x[j] && y == this.suspectInformation[i].y[j]) {
                    clueType = 2;
                }
            }
        }
        
        for (var i = 0; i < this.eliminationClues.length; i++) {
            for (var j = 0; j < this.eliminationClues[i].x.length; j++) {
                if (x == this.eliminationClues[i].x[j] && y == this.eliminationClues[i].y[j]) {
                    clueType = 3;
                }
            }
        }
        
        return clueType;
    };
    
    p.hasCompleteClueAt = function(x, y) {
        for (var i = 0; i < this.incriminatingEvidence.length; i++) {
            for (var j = 0; j < this.incriminatingEvidence[i].x.length; j++) {
                if (x == this.incriminatingEvidence[i].x[j] && y == this.incriminatingEvidence[i].y[j]) {
                    if (this.incriminatingEvidence[i].found == this.incriminatingEvidence[i].pieces) {
                        return true;
                    }
                }
            }
        }
        
        for (var i = 0; i < this.suspectInformation.length; i++) {
            for (var j = 0; j < this.suspectInformation[i].x.length; j++) {
                if (x == this.suspectInformation[i].x[j] && y == this.suspectInformation[i].y[j]) {
                    if (this.suspectInformation[i].found == this.suspectInformation[i].pieces) {
                        return true;
                    }
                }
            }
        }
        
        for (var i = 0; i < this.eliminationClues.length; i++) {
            for (var j = 0; j < this.eliminationClues[i].x.length; j++) {
                if (x == this.eliminationClues[i].x[j] && y == this.eliminationClues[i].y[j]) {
                    if (this.eliminationClues[i].found == this.eliminationClues[i].pieces) {
                        return true;
                    }
                }
            }
        }
        
        return false;
    };
    
    p.foundClueAt = function(x, y) {
        for (var i = 0; i < this.incriminatingEvidence.length; i++) {
            for (var j = 0; j < this.incriminatingEvidence[i].x.length; j++) {
                if (x == this.incriminatingEvidence[i].x[j] && y == this.incriminatingEvidence[i].y[j]) {
                    this.incriminatingEvidence[i].found++;
                }
            }
        }
        
        for (var i = 0; i < this.suspectInformation.length; i++) {
            for (var j = 0; j < this.suspectInformation[i].x.length; j++) {
                if (x == this.suspectInformation[i].x[j] && y == this.suspectInformation[i].y[j]) {
                    this.suspectInformation[i].found++;
                }
            }
        }
        
        for (var i = 0; i < this.eliminationClues.length; i++) {
            for (var j = 0; j < this.eliminationClues[i].x.length; j++) {
                if (x == this.eliminationClues[i].x[j] && y == this.eliminationClues[i].y[j]) {
                    this.eliminationClues[i].found++
                }
            }
        }
    };
    
    p.getClueAt = function(x, y) {
        for (var i = 0; i < this.incriminatingEvidence.length; i++) {
            for (var j = 0; j < this.incriminatingEvidence[i].x.length; j++) {
                if (x == this.incriminatingEvidence[i].x[j] && y == this.incriminatingEvidence[i].y[j]) {
                    return this.incriminatingEvidence[i];
                }
            }
        }
        
        for (var i = 0; i < this.suspectInformation.length; i++) {
            for (var j = 0; j < this.suspectInformation[i].x.length; j++) {
                if (x == this.suspectInformation[i].x[j] && y == this.suspectInformation[i].y[j]) {
                    return this.suspectInformation[i];
                }
            }
        }
        
        for (var i = 0; i < this.eliminationClues.length; i++) {
            for (var j = 0; j < this.eliminationClues[i].x.length; j++) {
                if (x == this.eliminationClues[i].x[j] && y == this.eliminationClues[i].y[j]) {
                    return this.eliminationClues[i];
                }
            }
        }
        
        return null;
    };
    
    p.getCompleteCount = function(clueType) {
        var count = 0;
        switch (clueType) {
            case 0:
                for (var i = 0; i < this.incriminatingEvidence.length; i++) {
                    if (this.incriminatingEvidence[i].found == this.incriminatingEvidence[i].pieces) {
                        count++;
                    }
                }
                return count;
            case 1:    
                for (var i = 0; i < this.suspectInformation.length; i++) {
                    if (this.suspectInformation[i].found == this.suspectInformation[i].pieces) {
                        count++;
                    }
                }
                return count;
            case 2: 
                for (var i = 0; i < this.eliminationClues.length; i++) {
                    if (this.eliminationClues[i].found == this.eliminationClues[i].pieces) {
                        count++;
                    }
                }
                return count;
        }
        
        return 0;
    };
    
    p.isInLevel = function(x, y) {
        var inside = true;
        
        if (x > this.width-2 || x < 1 || y > this.height-2 || y < 1) {
            inside = false;
        }
        
        return inside;
    };
    
window.Level = Level;
}(window));