//vars
var canvas;
var stage;

//usefull keycodes
var KEYCODE_SPACE = 32;        
var KEYCODE_UP = 38;
var KEYCODE_DOWN = 40;
var KEYCODE_LEFT = 37;
var KEYCODE_RIGHT = 39;
var KEYCODE_W = 87;
var KEYCODE_A = 65;
var KEYCODE_S = 83;
var KEYCODE_D = 68;

//asset dirs
var GFX_ROOT = './gfx/';
var SFX_ROOT = createjs.SoundJS.BASE_PATH = './sfx/';

//screensize setup
var allowResizing = false;
var canvasWidth = 960;
var canvasHeight = 640;

var PIXEL_RATIO = (function () {
var ctx = document.createElement("canvas").getContext("2d"),
    dpr = window.devicePixelRatio || 1,
    bsr = ctx.webkitBackingStorePixelRatio ||
          ctx.mozBackingStorePixelRatio ||
          ctx.msBackingStorePixelRatio ||
          ctx.oBackingStorePixelRatio ||
          ctx.backingStorePixelRatio || 1;
    return dpr / bsr;
})();

function createHiDPICanvas(w, h, ratio) {
    if (!ratio) { ratio = PIXEL_RATIO; }
    var can = document.createElement("canvas");
    can.width = w * ratio;
    can.height = h * ratio;
    can.style.width = w + "px";
    can.style.height = h + "px";
    can.getContext("2d").setTransform(ratio, 0, 0, ratio, 0, 0);
    return can;
};

//preloading setup
function load() {
    var canvasHolder = document.createElement('div');
    canvasHolder.id = 'canvasHolder';
    canvasHolder.style.width = '100%';
    canvasHolder.style.height = '100%';
    canvasHolder.style.margin = 'auto';
    canvasHolder.style.backgroundColor = '#000000';
    canvasHolder.style.textAlign = 'center';
    document.getElementById('gameDiv').appendChild(canvasHolder);

    var windowWidth = Math.min(window.innerWidth, 4096);
    var windowHeight = Math.min(window.innerHeight, 3072);
    windowWidth *= window.devicePixelRatio;
    windowHeight *= window.devicePixelRatio;
    
    //canvas = document.createElement('canvas');
    //canvas.width = canvasWidth;
    //canvas.height = canvasHeight;
    
    canvas = createHiDPICanvas(canvasWidth, canvasHeight);
    
    canvas.id = 'gameCanvas';
    
    if (allowResizing) {
        var verticalScale = (windowHeight / canvasHeight);
        var horizontalScale = (windowWidth / canvasWidth);
        
        if (verticalScale < horizontalScale) {
            canvas.style.width = canvasWidth * verticalScale + 'px';
            canvas.style.height = windowHeight + 'px';
        }
        else {
            canvas.style.width = windowWidth + 'px';
            canvas.style.height = canvasHeight * horizontalScale + 'px';
        }
    }
    else {
        canvas.style.width = canvasWidth + 'px';
        canvas.style.height = canvasHeight + 'px';
    }
    
    if (navigator.isCocoonJS) {
        canvas.style.cssText = 'idtkscale:ScaleToFill;';
    }
    
    document.getElementById('canvasHolder').appendChild(canvas);
    
	stage = new createjs.Stage(canvas);
    
    init();
};

//game setup
function init() {
    console.log("init from main");
    
    //create the engine
    var engine = new GameEngine(stage, canvasWidth, canvasHeight, 60); //60 fps
    
    //create the game state
    var game = new Game();
    
    //enable pixel snap
    //engine.enablePixelSnap();
    
    //show FPS
    //engine.showFPS();
    
    //add scenes
    engine.addScene( new PlayScene("PlayScene", game) );
    
    //set first scene
    engine.setScene("PlayScene");
    
    //start game engine
    engine.start();
    
    //enable keyboard input
    //engine.enableInput();
    
    //enable mouse over
    engine.enableMouse();
    
    //enable touch input
    engine.enableTouch();
    
    //enable window resize
    if (allowResizing) {
        engine.enableResize();
    }
    
};

//do not navigate away
window.onbeforeunload = function() {
    //return "Careful now!"; 
};

//start preloading immediatly
window.onload = load;
