(function(window) {

var Utility = function() {
	throw "Utility cannot be instantiated";
}

// private properties:
	Utility._dummy = 0;
    
// static methods:

    //scale an image using nearest neighbor sampling
	Utility.nearestNeighborScale = function(image, scale) {
        scale = Math.floor(scale);
        
        var sourceCanvas = document.createElement('canvas');
        sourceCanvas.width = image.width;
        sourceCanvas.height = image.height;
        sourceCanvas.getContext('2d').drawImage(image, 0, 0);
        var sourceData = sourceCanvas.getContext('2d').getImageData(0, 0, image.width, image.height).data;
        
        var scaledCanvas = document.createElement('canvas');
        scaledCanvas.width = image.width * scale;
        scaledCanvas.height = image.height * scale;
        
        var offset = 0;
        for (var y = 0; y < image.height; ++y) {
            for (var x = 0; x < image.width; ++x) {
                var r = sourceData[offset++];
                var g = sourceData[offset++];
                var b = sourceData[offset++];
                var a = sourceData[offset++] / 255;
                scaledCanvas.getContext('2d').fillStyle = 'rgba(' + r + ',' + g + ',' + b + ',' + a + ')';
                scaledCanvas.getContext('2d').fillRect(x * scale, y * scale, scale, scale);
            }
        }

        return scaledCanvas;
    };
    
    //returns a random number within a range
    Utility.random = function(min, max) {
        return min + Math.floor( Math.random() * ((max+1)-min) );
    };
    
    //returns random number between 0 and length of supplied weighted array
    Utility.weightedRandom = function(probabilities) {
        var total = 0;
        for (var j = 0; j < probabilities.length; j++) {
            total += probabilities[j];
        }
        
        var chance = Math.random() * total;
        var sum = 0;
        for (var i = 0; i < probabilities.length; i++) {
            sum += probabilities[i];
            if (chance < sum){
                return i;
            }
        }
        
        return probabilities.length-1;
    };
    
    //returns an approximated distance
    Utility.distance = function(dx, dy) {
        var min, max = 0;

        if (dx < 0) {
            dx = -dx;
        }
        if (dy < 0) {
            dy = -dy;
        }

        if (dx < dy) {
            min = dx;
            max = dy;
        } 
        else {
            min = dy;
            max = dx;
        }

        return ((( max << 8 ) + ( max << 3 ) - ( max << 4 ) - ( max << 1 ) + ( min << 7 ) - ( min << 5 ) + ( min << 3 ) - ( min << 1 )) >> 8 );
    };
    
    Utility.B1 = function(t){ return t*t*t; };
    Utility.B2 = function(t){ return 3*t*t*(1-t); };
    Utility.B3 = function(t){ return 3*t*(1-t)*(1-t); };
    Utility.B4 = function(t){ return (1-t)*(1-t)*(1-t); };
    
    Utility.getBezier = function(percent, p1, p2, p3, p4) {
        var pos = new createjs.Point();
        percent = 1.0 - percent;
        pos.x = p1.x*Utility.B1(percent) + p2.x*Utility.B2(percent) + p3.x*Utility.B3(percent) + p4.x*Utility.B4(percent);
        pos.y = p1.y*Utility.B1(percent) + p2.y*Utility.B2(percent) + p3.y*Utility.B3(percent) + p4.y*Utility.B4(percent);
        return pos;
    };
    
    Utility.getCatmull = function(t, p1, p2, p3, p4) {
        var pos = new createjs.Point();
        var t2 = t * t;
        var t3 = t2 * t;
        pos.x = 0.5 * ( ( 2.0 * p2.x ) + ( -p1.x + p3.x ) * t + ( 2.0 * p1.x - 5.0 * p2.x + 4 * p3.x - p4.x ) * t2 + ( -p1.x + 3.0 * p2.x - 3.0 * p3.x + p4.x ) * t3 );
        pos.y = 0.5 * ( ( 2.0 * p2.y ) + ( -p1.y + p3.y ) * t + ( 2.0 * p1.y - 5.0 * p2.y + 4 * p3.y - p4.y ) * t2 + ( -p1.y + 3.0 * p2.y - 3.0 * p3.y + p4.y ) * t3 );
        return pos;
    };
    
    Utility.isOdd = function(num) { 
        return (num % 2) != 0;
    };
    
    Utility.componentToHex = function(c) {
        var hex = c.toString(16);
        return hex.length == 1 ? "0" + hex : hex;
    }

    Utility.rgbToHex = function(r, g, b) {
        return "#" + Utility.componentToHex(r) + Utility.componentToHex(g) + Utility.componentToHex(b);
    };
    
    Utility.hexToRgb = function(hex) {
        var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
        return result ? {
            r: parseInt(result[1], 16),
            g: parseInt(result[2], 16),
            b: parseInt(result[3], 16)
        } : null;
    };
    
    Utility.hexToHexValue = function(hex) {
        var c = Utility.hexToRgb(hex);
        return (c.r << 16) + (c.g << 8) + (c.b);
    };
    
    Utility.hexValueToMatrix = function(hex, alpha) {
        return [ ((hex & 0x00FF0000) >>> 16) / 255 /*red*/, 0, 0, 0 
      	, 0, ((hex & 0x0000FF00) >>> 8) / 255 /*green*/, 0, 0
        , 0, 0, (hex & 0x000000FF) / 255 /*blue*/, 0
        , 0, 0, 0, (alpha || 100) / 100 ];
    };
    
    //Returns a dot formatted string
    Utility.dotFormat = function(number) {
        var numberString = number.toString();
        var dottedString = "";
        
        while (numberString.length > 3) {
            var part = numberString.substr(-3);
            numberString = numberString.substr(0, numberString.length - 3);
            dottedString = '.' + part + dottedString;
        }
        
        if (numberString.length > 0)
            dottedString = numberString + dottedString;
        
        return dottedString;
    };
    
    //Returns a time formatted string
    Utility.timeFormat = function(timeInSeconds) {
        var hours = Math.floor( timeInSeconds / (60*60));
        var minutes = Math.floor( (timeInSeconds-(hours*60*60)) / 60);
        var seconds = timeInSeconds -(hours*60*60) - (minutes*60);
        //console.log("Milliseconds: " + (timeInSeconds - (hours*60*60) - (minutes*60) - seconds) );
        var timeString = (String(hours).length == 1 ? "0":"") + hours + ":" + (String(minutes).length == 1 ? "0":"") + minutes + ":" + (String(seconds).length == 1 ? "0":"") + seconds;
        
        return timeString;
    }
        
window.Utility = Utility;
}(window));